import React from 'react';
import { createRoot } from 'react-dom/client';
import App from './App.tsx';
import './index.css';

// Analytics module with privacy focus
const Analytics = {
  initialized: false,
  
  init() {
    if (this.initialized) return;
    this.initialized = true;
    
    // Collect performance metrics
    if ('serviceWorker' in navigator && process.env.NODE_ENV === 'production') {
      window.addEventListener('load', () => {
        // Wait for page to fully load before collecting metrics
        setTimeout(() => {
          const navigationEntries = performance.getEntriesByType('navigation');
          const navigationEntry = navigationEntries[0] as PerformanceNavigationTiming;
          const paintEntries = performance.getEntriesByType('paint');
          
          const metrics = {
            // Core Web Vitals
            LCP: this.getLargestContentfulPaint(),
            FID: this.getFirstInputDelay(),
            CLS: this.getCumulativeLayoutShift(),
            // Navigation timing
            ttfb: navigationEntry ? navigationEntry.responseStart - navigationEntry.requestStart : undefined,
            domLoad: navigationEntry ? navigationEntry.domContentLoadedEventEnd - navigationEntry.startTime : undefined,
            windowLoad: navigationEntry ? navigationEntry.loadEventEnd - navigationEntry.startTime : undefined,
            // Paint timing
            firstPaint: this.getMetricFromPerformanceEntries(paintEntries, 'first-paint'),
            firstContentfulPaint: this.getMetricFromPerformanceEntries(paintEntries, 'first-contentful-paint'),
            // Page info
            url: window.location.href,
            referrer: document.referrer,
            userAgent: navigator.userAgent,
            timestamp: Date.now(),
          };
          
          // Send metrics to service worker (privacy-preserving)
          if (navigator.serviceWorker.controller) {
            navigator.serviceWorker.controller.postMessage({
              type: 'RECORD_METRICS',
              payload: metrics
            });
          }
        }, 3000);
      });
    }
  },
  
  getLargestContentfulPaint() {
    const entries = performance.getEntriesByType('element') as PerformanceElementTiming[];
    let maxSize = 0;
    let lcpValue;
    
    entries.forEach(entry => {
      if (entry.size > maxSize) {
        maxSize = entry.size;
        lcpValue = entry.startTime;
      }
    });
    
    return lcpValue;
  },
  
  getFirstInputDelay() {
    // Use first-input entries
    const fiEntries = performance.getEntriesByType('first-input') as PerformanceEventTiming[];
    if (fiEntries && fiEntries.length > 0) {
      return fiEntries[0].processingStart - fiEntries[0].startTime;
    }
    
    return undefined;
  },
  
  getCumulativeLayoutShift() {
    // Use layout-shift entries
    let cumulativeScore = 0;
    const entries = performance.getEntriesByType('layout-shift') as LayoutShiftAttribution[];
    
    entries.forEach(entry => {
      if (!entry.hadRecentInput) {
        cumulativeScore += entry.value;
      }
    });
    
    return cumulativeScore;
  },
  
  getMetricFromPerformanceEntries(entries: PerformanceEntry[], name: string) {
    const entry = entries.find(entry => entry.name === name);
    return entry ? entry.startTime : undefined;
  }
};

// Remove debug code in production
if (process.env.NODE_ENV === 'production') {
  console.log = () => {};
  console.warn = () => {};
  console.error = (msg) => {
    // Keep error tracking in production but without details
    window.onerror && window.onerror(msg);
  };
}

// Performance measurement
const startTime = performance.now();

// Find root element
const rootElement = document.getElementById("root");
if (!rootElement) throw new Error('Root element not found');

// Use createRoot API for better performance
const root = createRoot(rootElement);

// Force initial scroll position to top
window.history.scrollRestoration = "manual";

// Prefetch popular routes that may be visited next
const prefetchRoutes = () => {
  const routes = [
    '/privacy-policy', 
    '/terms-of-service'
  ];
  
  if ('requestIdleCallback' in window) {
    window.requestIdleCallback(() => {
      routes.forEach(route => {
        const link = document.createElement('link');
        link.rel = 'prefetch';
        link.href = route;
        link.as = 'document';
        document.head.appendChild(link);
      });
    });
  }
};

// Use a RequestIdleCallback to render when browser is idle
// Fall back to setTimeout for browsers that don't support it
const scheduleRender = window.requestIdleCallback || 
  ((cb) => setTimeout(cb, 1));

scheduleRender(() => {
  // Render app
  root.render(
    <React.StrictMode>
      <App />
    </React.StrictMode>
  );
  
  // Log render time in development only
  if (process.env.NODE_ENV !== 'production') {
    const renderTime = performance.now() - startTime;
    console.log(`Initial render time: ${renderTime.toFixed(2)}ms`);
  }
  
  // Initialize analytics in production
  if (process.env.NODE_ENV === 'production') {
    Analytics.init();
  }
  
  // Prefetch routes after app is rendered
  prefetchRoutes();
});

// Register service worker for better performance
if ('serviceWorker' in navigator && process.env.NODE_ENV === 'production') {
  window.addEventListener('load', () => {
    navigator.serviceWorker.register('/service-worker.js', {
      scope: '/'
    }).then(registration => {
      console.log('ServiceWorker registration successful');
    }).catch(error => {
      console.error('ServiceWorker registration failed:', error);
    });
  });
}

// Preload key resources
const preloadResources = () => {
  // Only preload in production
  if (process.env.NODE_ENV !== 'production') return;
  
  const resources = [
    { href: 'https://media.publit.io/file/Enavra/Enavra-Hero-Background-1920x1200.mp4', as: 'video' }
  ];
  
  resources.forEach(resource => {
    const link = document.createElement('link');
    link.rel = 'preload';
    link.href = resource.href;
    link.as = resource.as;
    document.head.appendChild(link);
  });
};

// Run preload after the page has loaded
window.addEventListener('load', () => {
  // Use requestIdleCallback for non-critical operations
  if (window.requestIdleCallback) {
    window.requestIdleCallback(preloadResources);
  } else {
    setTimeout(preloadResources, 1000);
  }
});

// Add TypeScript interfaces for different performance entry types
interface LayoutShiftAttribution extends PerformanceEntry {
  value: number;
  hadRecentInput: boolean;
}

interface PerformanceElementTiming extends PerformanceEntry {
  size: number;
}

interface PerformanceEventTiming extends PerformanceEntry {
  processingStart: number;
}
